﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.IO.Ports;
using System.Net;
using System.Threading;
using System.Xml;
using Timer = System.Timers.Timer;

namespace W13WpfApp
{
    /// <summary>
    /// Logique d'interaction pour MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private string comPortToUse = "COM5";
        private int baudRate = 9600;

        private bool portFound;
        private SerialPort sport;

        private bool holdCW;
        private bool holdCCW;

        private Timer holdTimer;

        public MainWindow()
        {
            InitializeComponent();

            holdTimer = new Timer(1000);
            holdTimer.Elapsed += (sender, a) => Turn();

            try
            {
                if (sport == null || !sport.IsOpen)
                {
                    while (!GetSerialPorts())
                    {
                        Thread.Sleep(5000);
                    }
                }
                else
                {
                    Thread.Sleep(3000);
                }
            }
            finally
            {
            }
        }

        private void Turn()
        {
            string direction = "a";
            if (holdCW)
            {
                direction = "b";
            }

            if (holdCCW)
            {
                direction = "c";
            }

            if (holdCW || holdCCW)
            {
                try
                {
                    byte[] hexstring = Encoding.ASCII.GetBytes(direction);
                    foreach (byte hexval in hexstring)
                    {
                        byte[] _hexval = new byte[] { hexval }; // need to convert byte to byte[] to write
                        sport.Write(_hexval, 0, 1);
                        Thread.Sleep(1);
                    }
                }
                catch (Exception ex)
                {
                    Console.WriteLine("Failed to SEND \n" + ex + "\n");

                }
            }            
        }

        private bool GetSerialPorts()
        {
            foreach (var portName in SerialPort.GetPortNames())
            {
                Console.WriteLine("FOUND " + portName);
            }

            portFound = SerialPort.GetPortNames().Contains(comPortToUse);

            if (!portFound)
            {
                Console.WriteLine(comPortToUse + " port not found !");
                return false;
            }

            serialport_connect(comPortToUse, baudRate);

            return true;
        }

        public void serialport_connect(String port, int baudrate)
        {
            sport = new SerialPort(port, baudrate);
            sport.PortName = port;
            sport.BaudRate = Convert.ToInt32(baudrate);
            sport.Handshake = System.IO.Ports.Handshake.None;
            sport.Parity = Parity.None;
            sport.DataBits = 8;
            sport.StopBits = StopBits.Two;
            sport.ReadTimeout = 200;
            sport.WriteTimeout = 50;

            try
            {
                sport.Open();
                sport.DataReceived += SportDataReceived;
                Console.WriteLine("COM port open");
                CwButton.IsEnabled = true;
                CcwButton.IsEnabled = true;
            }
            catch (Exception ex)
            {
                Console.WriteLine("EXCEPTION : " + ex, "Error");
            }

            holdTimer.Start();
        }

        string readValue;

        private void SportDataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            DateTime dt = DateTime.Now;
            String dtn = dt.ToShortTimeString();
            readValue = sport.ReadExisting();

            Dispatcher.BeginInvoke(new Action(delegate ()
            {
                rx.Text = readValue;
            }));
           

            Console.WriteLine("[" + dtn + "] " + "Received: " + readValue + "\n");
        }

        public void ClosePort()
        {
            try
            {
                holdTimer.Stop();
                if (sport.IsOpen)
                {
                    sport.Close();
                    Console.WriteLine("COM port closed");
                }
            }
            catch (Exception)
            {
                Console.WriteLine();
            }
        }


        public static string TextToBase64(string sAscii)
        {
            byte[] bytes = Encoding.ASCII.GetBytes(sAscii);
            return System.Convert.ToBase64String(bytes, 0, bytes.Length);
        }


        private void CCW_Button_Clicked(object sender, RoutedEventArgs e)
        {
            if (sport.IsOpen)
            {

            }
        }

        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            ClosePort();
        }

        private void CWButton_MouseDown(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            if (sport.IsOpen)
            {
                holdCW = true;
                Console.WriteLine("holding");
            }
        }

        private void CWButton_MouseUp(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            if (sport.IsOpen)
            {
                holdCW = false;
                Console.WriteLine("releasing");
            }
        }

        private void CCWButton_MouseDown(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            if (sport.IsOpen)
            {
                holdCCW = true;
                Console.WriteLine("holding");
            }
        }

        private void CCWButton_MouseUp(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            if (sport.IsOpen)
            {
                holdCCW = false;
                Console.WriteLine("releasing");
            }
        }


    }
}
